/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.sejda.sambox.encryption;

import java.io.InputStream;

/**
 * Encryption algorithm with AES engine
 * 
 * @author Andrea Vacondio
 *
 */
interface AESEncryptionAlgorithmEngine extends EncryptionAlgorithmEngine
{
    /**
     * @param data
     * @param key
     * @param iv
     * @return a stream encrypted with the given key and initialization vector
     */
    InputStream encryptStream(InputStream data, byte[] key, byte[] iv);

    /**
     * 
     * @param data
     * @param key
     * @param iv
     * @return the byte array encrypted with the given key and initialization vector
     */
    byte[] encryptBytes(byte[] data, byte[] key, byte[] iv);
}
